#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define M 46 
#define N 16 

void v_add(int *v1, int *v2, int n)
{
	int i;

	for (i = 0; i < n; i++)
		v1[i] += v2[i];
}

void v_show(int *v, int n)
{
	int i;

	for (i = 0; i < n; i++)
		printf("%2d ", v[i]);	
	printf("\n");
}

double prob_r(int *y, int *z, int n)
{
	int i;	
	double r;

	r = 1.0;
	for (i = 0; i < n; i++)
		if (z[i] == 1) {
			r /= (y[i] + 1);
		} else if (z[i] == -1) {
			r *= y[i];
		}
	return r;
}

double chisq(int *y, double *yy, int n)
{
        int i;
        double x = 0.0;

        for (i = 0; i < n; i++)
                x += (y[i]-yy[i])*(y[i]-yy[i])/yy[i];
        return x;
}

double *fitted_v(double *rs, double *cs, int sum, int m)
{
        int i, j;
        double *v;

        v = (double *)malloc(sizeof(double) * m * m);
        for (i = 0; i < m; i++)
                for (j = 0; j < m; j++)
                        v[m * i + j] = rs[i] * cs[j]/sum;
        return v;
}

double likelihood_ratio(int *y, double *mu, int n)
{
	int i;
	double x = 0.0;

	for (i = 0; i < n; i++)
		x += y[i] * log(y[i]/mu[i]);
	return 2 * x;
}

int main(int argc, char *argv[])
{
	int i, j;
	int seed = 0;
	int n = 10000;
	int burnin = n;
	double r;
	double rr;
	int markov_b[M][N] = {
{-1,-1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0,-1,-1, 0, 0 },
{-1,-1, 0, 1, 1, 1,-1, 0, 1, 0, 0, 0, 0,-1, 0, 0 },
{-1,-1, 1, 0, 1, 1, 0,-1, 0, 1, 0, 0,-1, 0, 0, 0 },
{-1,-1, 1, 1, 1, 1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0 },
{-1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0,-1 },
{-1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0,-1,-1,-1, 1, 0 },
{-1, 0, 0, 1, 0, 1,-1, 0, 1, 0, 0,-1, 0,-1, 1, 0 },
{-1, 0, 1, 0, 0, 1, 0,-1, 0, 1, 0,-1,-1, 0, 1, 0 },
{-1, 0, 1, 0, 1, 0,-1, 0, 1, 0,-1, 0,-1, 0, 1, 0 },
{-1, 0, 1, 1, 0, 1,-1,-1, 0, 0, 0,-1, 0, 0, 1, 0 },
{ 0,-1, 0, 0, 1, 0, 0, 0, 1, 1,-1, 0,-1,-1, 0, 1 },
{ 0,-1, 0, 1, 0, 1, 0,-1, 0, 1, 0,-1, 0,-1, 0, 1 },
{ 0,-1, 0, 1, 1, 0,-1, 0, 1, 0,-1, 0, 0,-1, 0, 1 },
{ 0,-1, 1, 0, 1, 0, 0,-1, 0, 1,-1, 0,-1, 0, 0, 1 },
{ 0,-1, 1, 1, 1, 0,-1,-1, 0, 0,-1, 0, 0, 0, 0, 1 },
{ 0, 0,-1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0,-1, 0, 0 },
{ 0, 0, 0,-1, 0, 0, 1, 0, 0, 1, 0, 0,-1, 0, 0, 0 },
{ 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,-1,-1,-1,-1, 1, 1 },
{ 0, 0, 0, 1, 0, 0,-1, 0, 1, 0,-1,-1, 0,-1, 1, 1 },
{ 0, 0, 1, 0, 0, 0, 0,-1, 0, 1,-1,-1,-1, 0, 1, 1 },
{ 0, 0, 1, 1, 0, 0,-1,-1, 0, 0,-1,-1, 0, 0, 1, 1 },
{ 0, 1, 0, 0,-1, 0, 0, 0, 0, 0, 0,-1, 0, 0, 1, 0 },
{ 1, 0,-1, 1,-1, 0, 0, 0, 0, 0, 0,-1, 1,-1, 0, 1 },

{ 1, 1, 0, 0,-1,-1, 0, 0,-1,-1, 0, 0, 1, 1, 0, 0 },
{ 1, 1, 0,-1,-1,-1, 1, 0,-1, 0, 0, 0, 0, 1, 0, 0 },
{ 1, 1,-1, 0,-1,-1, 0, 1, 0,-1, 0, 0, 1, 0, 0, 0 },
{ 1, 1,-1,-1,-1,-1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0 },
{ 1, 0, 0, 0, 0,-1, 0, 0, 0, 0,-1, 0, 0, 0, 0, 1 },
{ 1, 0, 0, 0, 0,-1, 0, 0,-1,-1, 0, 1, 1, 1,-1, 0 },
{ 1, 0, 0,-1, 0,-1, 1, 0,-1, 0, 0, 1, 0, 1,-1, 0 },
{ 1, 0,-1, 0, 0,-1, 0, 1, 0,-1, 0, 1, 1, 0,-1, 0 },
{ 1, 0,-1, 0,-1, 0, 1, 0,-1, 0, 1, 0, 1, 0,-1, 0 },
{ 1, 0,-1,-1, 0,-1, 1, 1, 0, 0, 0, 1, 0, 0,-1, 0 },
{ 0, 1, 0, 0,-1, 0, 0, 0,-1,-1, 1, 0, 1, 1, 0,-1 },
{ 0, 1, 0,-1, 0,-1, 0, 1, 0,-1, 0, 1, 0, 1, 0,-1 },
{ 0, 1, 0,-1,-1, 0, 1, 0,-1, 0, 1, 0, 0, 1, 0,-1 },
{ 0, 1,-1, 0,-1, 0, 0, 1, 0,-1, 1, 0, 1, 0, 0,-1 },
{ 0, 1,-1,-1,-1, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0,-1 },
{ 0, 0, 1, 0, 0, 0, 0,-1,-1, 0, 0, 0, 0, 1, 0, 0 },
{ 0, 0, 0, 1, 0, 0,-1, 0, 0,-1, 0, 0, 1, 0, 0, 0 },
{ 0, 0, 0, 0, 0, 0, 0, 0,-1,-1, 1, 1, 1, 1,-1,-1 },
{ 0, 0, 0,-1, 0, 0, 1, 0,-1, 0, 1, 1, 0, 1,-1,-1 },
{ 0, 0,-1, 0, 0, 0, 0, 1, 0,-1, 1, 1, 1, 0,-1,-1 },
{ 0, 0,-1,-1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0,-1,-1 },
{ 0,-1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0,-1, 0 },
{-1, 0, 1,-1, 1, 0, 0, 0, 0, 0, 0, 1,-1, 1, 0,-1 }
	};

	int state[N] = {
	69, 31, 55, 149, 46, 43, 118, 30, 43, 45, 71, 380, 37, 36, 212, 52
	};
	double sum = 0.0;


	double mu[N] = {
 64.52677, 47.25345, 53.14603, 151.07960, 30.42595, 46.79383, 115.24100, 32.53337,
 49.42430, 46.13193, 70.90290, 360.53502, 35.18867, 30.25510, 232.14438, 51.41770
 	};
	double lratio;
	double tlratio;
        int tlratio_count = 0;

	if (argc >= 2) {
		seed = atoi(argv[1]);	
	} 
	if (argc >= 3) {
		n = atoi(argv[2]);	
	} 
	if (argc >= 4) {
		burnin = atoi(argv[3]);
	}

	lratio = likelihood_ratio(state, mu, N);
        printf("likelihood ratio statistic : %lf\n", lratio);

	srand(seed);

	/* burn-in */
	for (i = 0; i < burnin; i++) {
		j = (int)(M * (rand()/(RAND_MAX + 1.0)));
		r = prob_r(state, markov_b[j], N);	
		rr = (double)rand()/RAND_MAX;	
		if (r > rr)
			v_add(state, markov_b[j], N);
	}

	for (i = 0; i < n; i++) {
		j = (int)(M * (rand() / (RAND_MAX + 1.0))); 
		r = prob_r(state, markov_b[j], N);	
		rr = (double)rand()/RAND_MAX;	
		if (r > rr)
			v_add(state, markov_b[j], N);
		tlratio = likelihood_ratio(state, mu, N);
		if (tlratio > lratio) {
			tlratio_count++;
			printf("* ");
		}
		v_show(state, N);
	}
	printf("tlratio_count : %d\n", tlratio_count);
	printf("appoximate p-value : %lf\n", (double)tlratio_count/n);

	return 0;
}

