#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define N 3

double prob_(int *state, double theta)
{
	return exp(theta*(state[0]*state[1] + state[1]*state[2] + state[0]*state[2]));
}

double z(double theta)
{
	return 2.0*exp(3.0*theta) + 6.0*exp(-theta);
}

double prob(int *state, double theta)
{
	return prob_(state, theta) / z(theta);
}

void show_state(int *state, int n)
{
	int i;

	for (i = 0; i < n; i++)
		printf("%2d ", state[i]);
	printf("\n");
}

void cpy_state(int *from, int *to, int n)
{
	int i;

	for (i = 0; i < n; i++)
		to[i] = from[i];
}

int gen_index(int *state, int n)
{
	int i;
	int index = 0;
	int t;

	for (i = 0; i < n; i++) {
		if (state[i] == -1)	
			t = 0;
		else
			t = 1;
		index = 2 * index + t;
	}
	return index;
}

int main(int argc, char *argv[])
{
	int i = 0;
	int j;
	int seed = 0;
	int n = 10000;
	int state[N] = {1,1,1};
	int old_state[N];
	double r;
	double theta = 0.2;
	int hist[8];
	int total;
	int allstate[8][3] = {
		{-1,-1,-1},{-1,-1,1},{-1,1,-1},{-1,1,1},
		{1,-1,-1},{1,-1,1},{1,1,-1},{1,1,1}};
	int burnin = n;

	if (argc >= 2) {
		seed = atoi(argv[1]);	
	} 
	if (argc >= 3) {
		n = atoi(argv[2]);	
	} 
	if (argc >= 4) {
		burnin = atoi(argv[3]);
	} 


	srand(seed);
	for (i = 0; i < 8; i++)
		hist[i] = 0;


	for (i = 0; i < n; i++) {
		cpy_state(state, old_state, N);
		j = rand() % 3;
		state[j] = -state[j];
		r = prob_(state, theta)/prob_(old_state, theta);
		if (r <= (double)rand()/RAND_MAX) 
			state[j] = -state[j];
	}

	for (i = 0; i < n; i++) {
		cpy_state(state, old_state, N);
		j = rand() % 3;
		state[j] = -state[j];
		r = prob_(state, theta)/prob_(old_state, theta);
		if (r <= (double)rand()/RAND_MAX) 
			state[j] = -state[j];
		show_state(state, N);
		hist[gen_index(state, N)]++; 
	}

	for (i = 0, total = 0; i < 8; i++) {
		printf("index %d : %d\n", i, hist[i]);
		total += hist[i];
	}
	printf("total : %d\n", total); 
	printf("z : %f\n", z(theta));
	for (i = 0; i < 8; i++) {
		printf("index %d : %f %f\n", i, (float)hist[i] / total,
		prob(allstate[i], theta));
	}

	return 0;
}
